@@ -3,7 +3,7 @@ cclib (http://cclib.sf.net) is (c) 2006, the cclib development team
 and licensed under the LGPL (http://www.gnu.org/copyleft/lgpl.html).
 """
 
-__revision__ = "$Revision: 713 $"
+__revision__ = "$Revision: 834 $"
 
 
 import re
@@ -17,10 +17,10 @@ import logfileparser
 class GAMESS(logfileparser.Logfile):
     """A GAMESS log file."""
     SCFRMS, SCFMAX, SCFENERGY = range(3) # Used to index self.scftargets[]
-    def __init__(self, *args):
+    def __init__(self, *args, **kwargs):
 
         # Call the __init__ method of the superclass
-        super(GAMESS, self).__init__(logname="GAMESS", *args)
+        super(GAMESS, self).__init__(logname="GAMESS", *args, **kwargs)
 
     def __str__(self):
         """Return a string representation of the object."""
@@ -145,18 +145,28 @@ class GAMESS(logfileparser.Logfile):
             self.mult = int(inputfile.next().split()[-1])
 
         # etenergies (used only for CIS runs now)
-        if "EXCITATION ENERGIES" in line:
+        if "EXCITATION ENERGIES" in line and line.find("DONE WITH") < 0:
             if not hasattr(self, "etenergies"):
                 self.etenergies = []
-            header = inputfile.next()
+            header = inputfile.next().rstrip()
+            get_etosc = False
+            if header.endswith("OSC. STR."):
+                # water_cis_dets.out does not have the oscillator strength
+                # in this table...it is extracted from a different section below
+                get_etosc = True
+                self.etoscs = []
             dashes = inputfile.next()
             line = inputfile.next()
-            while len(line.split()) > 0:
+            broken = line.split()
+            while len(broken) > 0:
                 # Take hartree value with more numbers, and convert.
                 # Note that the values listed after this are also less exact!
-                etenergy = float(line.split()[1])
+                etenergy = float(broken[1])
                 self.etenergies.append(utils.convertor(etenergy, "hartree", "cm-1"))
-                line = inputfile.next()
+                if get_etosc:
+                    etosc = float(broken[-1])
+                    self.etoscs.append(etosc)
+                broken = inputfile.next().split()
 
         # Detect the CI hamiltonian type, if applicable.
         # Should always be detected if CIS is done.
@@ -196,12 +206,12 @@ class GAMESS(logfileparser.Logfile):
                 # With the SAPS hamiltonian, the coefficients are multiplied
                 #   by sqrt(2) so that they normalize to 1.
                 # With DETS, both alpha and beta excitations are printed.
-                if self.cihamtyp == "saps":
-                    coeff /= numpy.sqrt(2.0)
+                # if self.cihamtyp == "saps":
+                #    coeff /= numpy.sqrt(2.0)
                 CIScontribs.append([(fromMO,MOtype),(toMO,MOtype),coeff])
                 line = inputfile.next()
             self.etsecs.append(CIScontribs)
-            
+
         # etoscs (used only for CIS runs now)
         if line[1:50] == "TRANSITION FROM THE GROUND STATE TO EXCITED STATE":
             if not hasattr(self, "etoscs"):
@@ -213,6 +223,38 @@ class GAMESS(logfileparser.Logfile):
             strength = float(line.split()[3])
             self.etoscs.append(strength)
 
+        # TD-DFT for GAMESS-US
+        if line[14:29] == "LET EXCITATIONS": # TRIPLET and SINGLET
+            self.etenergies = []
+            self.etoscs = []
+            self.etsecs = []
+            etsyms = []
+            minus = inputfile.next()
+            blank = inputfile.next()
+            line = inputfile.next()
+            # Loop starts on the STATE line
+            while line.find("STATE") >= 0:
+                broken = line.split()
+                self.etenergies.append(utils.convertor(float(broken[-2]), "eV", "cm-1"))
+                broken = inputfile.next().split()
+                self.etoscs.append(float(broken[-1]))
+                sym = inputfile.next() # Not always present
+                if sym.find("SYMMETRY")>=0:
+                    etsyms.append(sym.split()[-1])
+                    header = inputfile.next()
+                minus = inputfile.next()
+                CIScontribs = []
+                line = inputfile.next()
+                while line.strip():
+                    broken = line.split()
+                    fromMO, toMO = [int(broken[x]) - 1 for x in [2, 4]]
+                    CIScontribs.append([(fromMO, 0), (toMO, 0), float(broken[1])])
+                    line = inputfile.next()
+                self.etsecs.append(CIScontribs)
+                line = inputfile.next()
+            if etsyms: # Not always present
+                self.etsyms = etsyms
+         
         # Maximum and RMS gradients.
         if "MAXIMUM GRADIENT" in line or "RMS GRADIENT" in line:
 
@@ -564,6 +606,10 @@ class GAMESS(logfileparser.Logfile):
             for base in range(0, self.nmo, 5):
 
                 line = inputfile.next()
+                # Make sure that this section does not end prematurely - checked by regression test 2CO.ccsd.aug-cc-pVDZ.out.
+                if line.strip() != "":
+                    break;
+                
                 numbers = inputfile.next() # Eigenvector numbers.
 
                 # Eigenvalues for these orbitals (in hartrees).
