@@ -3,7 +3,7 @@ cclib (http://cclib.sf.net) is (c) 2006, the cclib development team
 and licensed under the LGPL (http://www.gnu.org/copyleft/lgpl.html).
 """
 
-__revision__ = "$Revision: 717 $"
+__revision__ = "$Revision: 784 $"
 
 
 import re
@@ -17,10 +17,10 @@ import logfileparser
 class GAMESSUK(logfileparser.Logfile):
     """A GAMESS UK log file"""
     SCFRMS, SCFMAX, SCFENERGY = range(3) # Used to index self.scftargets[]
-    def __init__(self, *args):
+    def __init__(self, *args, **kwargs):
 
         # Call the __init__ method of the superclass
-        super(GAMESSUK, self).__init__(logname="GAMESSUK", *args)
+        super(GAMESSUK, self).__init__(logname="GAMESSUK", *args, **kwargs)
         
     def __str__(self):
         """Return a string representation of the object."""
