@@ -3,11 +3,10 @@ cclib (http://cclib.sf.net) is (c) 2006, the cclib development team
 and licensed under the LGPL (http://www.gnu.org/copyleft/lgpl.html).
 """
 
-__revision__ = "$Revision: 750 $"
+__revision__ = "$Revision: 844 $"
 
 
 import re
-
 import numpy
 
 import utils
@@ -17,10 +16,10 @@ import logfileparser
 class Gaussian(logfileparser.Logfile):
     """A Gaussian 98/03 log file."""
 
-    def __init__(self, *args):
+    def __init__(self, *args, **kwargs):
 
         # Call the __init__ method of the superclass
-        super(Gaussian, self).__init__(logname="Gaussian", *args)
+        super(Gaussian, self).__init__(logname="Gaussian", *args, **kwargs)
         
     def __str__(self):
         """Return a string representation of the object."""
@@ -63,6 +62,8 @@ class Gaussian(logfileparser.Logfile):
         self.optfinished = False
         # Flag for identifying Coupled Cluster runs.
         self.coupledcluster = False
+        # Fragment number for counterpoise calculations (normally zero).
+        self.counterpoise = 0
 
     def extract(self, inputfile, line):
         """Extract information from the file object inputfile."""
@@ -73,10 +74,7 @@ class Gaussian(logfileparser.Logfile):
             self.updateprogress(inputfile, "Attributes", self.fupdate)
                     
             natom = int(line.split()[1])
-            if hasattr(self, "natom"):
-                assert self.natom == natom
-            else:
-                # I wonder whether this code will ever be executed.
+            if not hasattr(self, "natom"):
                 self.natom = natom
 
         # Catch message about completed optimization.
@@ -87,6 +85,10 @@ class Gaussian(logfileparser.Logfile):
         #   in the event the standard orientation isn't available.
         if line.find("Input orientation") > -1 or line.find("Z-Matrix orientation") > -1:
 
+            # If this is a counterpoise calculation, this output means that
+            #   the supermolecule is now being considered, so we can set:
+            self.counterpoise = 0
+
             self.updateprogress(inputfile, "Attributes", self.cupdate)
                     
             self.inputcoords = []
@@ -115,7 +117,11 @@ class Gaussian(logfileparser.Logfile):
         if not self.optfinished and line[25:45] == "Standard orientation":
 
             self.updateprogress(inputfile, "Attributes", self.cupdate)
-                    
+
+            # If this is a counterpoise calculation, this output means that
+            #   the supermolecule is now being considered, so we can set:
+            self.counterpoise = 0
+
             if not hasattr(self, "atomcoords"):
                 self.atomcoords = []
             
@@ -314,15 +320,26 @@ class Gaussian(logfileparser.Logfile):
             self.geovalues.append(newlist)
 
         # Charge and multiplicity.
+        # If counterpoise correction is used, multiple lines match.
+        # The first one contains charge/multiplicity of the whole molecule.:
+        #   Charge =  0 Multiplicity = 1 in supermolecule
+        #   Charge =  0 Multiplicity = 1 in fragment  1.
+        #   Charge =  0 Multiplicity = 1 in fragment  2.
         if line[1:7] == 'Charge' and line.find("Multiplicity")>=0:
 
-            broken = line.split()
-            self.charge = int(broken[2])
-            self.mult = int(broken[-1])
+            regex = ".*=(.*)Mul.*=\s*(\d+).*"
+            match = re.match(regex, line)
+            assert match, "Something unusual about the line: '%s'" % line
+            
+            self.charge = int(match.groups()[0])
+            self.mult = int(match.groups()[1])
 
         # Orbital symmetries.
         if line[1:20] == 'Orbital symmetries:' and not hasattr(self, "mosyms"):
 
+            # For counterpoise fragments, skip these lines.
+            if self.counterpoise != 0: return
+
             self.updateprogress(inputfile, "MO Symmetries", self.fupdate)
                     
             self.mosyms = [[]]
@@ -358,41 +375,54 @@ class Gaussian(logfileparser.Logfile):
         # Alpha/Beta electron eigenvalues.
         if line[1:6] == "Alpha" and line.find("eigenvalues") >= 0:
 
+            # For counterpoise fragments, skip these lines.
+            if self.counterpoise != 0: return
+
             self.updateprogress(inputfile, "Eigenvalues", self.fupdate)
-                    
             self.moenergies = [[]]
             HOMO = -2
+
             while line.find('Alpha') == 1:
                 if line.split()[1] == "virt." and HOMO == -2:
-                    # If there aren't any symmetries,
-                    # this is a good way to find the HOMO
+
+                    # If there aren't any symmetries, this is a good way to find the HOMO.
+                    # Also, check for consistency if homos was already parsed.
                     HOMO = len(self.moenergies[0])-1
                     if hasattr(self, "homos"):
                         assert HOMO == self.homos[0]
                     else:
                         self.homos = numpy.array([HOMO], "i")
+
                 part = line[28:]
                 i = 0
                 while i*10+4 < len(part):
                     x = part[i*10:(i+1)*10]
                     self.moenergies[0].append(utils.convertor(self.float(x), "hartree", "eV"))
                     i += 1
-                line = inputfile.next()            
+                line = inputfile.next()
+            # If, at this point, self.homos is unset, then there were not
+            # any alpha virtual orbitals
+            if not hasattr(self, "homos"):
+                HOMO = len(self.moenergies[0])-1
+                self.homos = numpy.array([HOMO], "i")
+            
+
             if line.find('Beta') == 2:
                 self.moenergies.append([])
+
             HOMO = -2
             while line.find('Beta') == 2:
                 if line.split()[1] == "virt." and HOMO == -2:
-                    # If there aren't any symmetries,
-                    # this is a good way to find the HOMO
+
+                    # If there aren't any symmetries, this is a good way to find the HOMO.
+                    # Also, check for consistency if homos was already parsed.
                     HOMO = len(self.moenergies[1])-1
                     if len(self.homos) == 2:
-                        # It already has a self.homos (with the Alpha value)
-                        # but does it already have a Beta value?
                         assert HOMO == self.homos[1]
                     else:
                         self.homos.resize([2])
                         self.homos[1] = HOMO
+
                 part = line[28:]
                 i = 0
                 while i*10+4 < len(part):
@@ -400,21 +430,26 @@ class Gaussian(logfileparser.Logfile):
                     self.moenergies[1].append(utils.convertor(self.float(x), "hartree", "eV"))
                     i += 1
                 line = inputfile.next()
+
             self.moenergies = [numpy.array(x, "d") for x in self.moenergies]
             
+        # Gaussian Rev <= B.0.3 (?)
+        # AO basis set in the form of general basis input:
+        #  1 0
+        # S   3 1.00       0.000000000000
+        #      0.7161683735D+02  0.1543289673D+00
+        #      0.1304509632D+02  0.5353281423D+00
+        #      0.3530512160D+01  0.4446345422D+00
+        # SP   3 1.00       0.000000000000
+        #      0.2941249355D+01 -0.9996722919D-01  0.1559162750D+00
+        #      0.6834830964D+00  0.3995128261D+00  0.6076837186D+00
+        #      0.2222899159D+00  0.7001154689D+00  0.3919573931D+00
         if line[1:14] == "AO basis set ":
-            ## Gaussian Rev <= B.0.3
+        
+            # For counterpoise fragment calcualtions, skip these lines.
+            if self.counterpoise != 0: return
+        
             self.gbasis = []
-            # AO basis set in the form of general basis input:
-            #  1 0
-            # S   3 1.00       0.000000000000
-            #      0.7161683735D+02  0.1543289673D+00
-            #      0.1304509632D+02  0.5353281423D+00
-            #      0.3530512160D+01  0.4446345422D+00
-            # SP   3 1.00       0.000000000000
-            #      0.2941249355D+01 -0.9996722919D-01  0.1559162750D+00
-            #      0.6834830964D+00  0.3995128261D+00  0.6076837186D+00
-            #      0.2222899159D+00  0.7001154689D+00  0.3919573931D+00
             line = inputfile.next()
             while line.strip():
                 gbasis = []
@@ -448,7 +483,7 @@ class Gaussian(logfileparser.Logfile):
             while line.strip() != "":
 
                 # Lines with symmetries and symm. indices begin with whitespace.
-                if line[1:15].strip() == "" and not line[21].isdigit():
+                if line[1:15].strip() == "" and not line[15:22].strip().isdigit():
 
                     if not hasattr(self, 'vibsyms'):
                         self.vibsyms = []
@@ -492,7 +527,7 @@ class Gaussian(logfileparser.Logfile):
                             for n in range(N):
                                 disps.append([])
                         for n in range(N):
-                            disps[n].append(numbers[n:n+3])
+                            disps[n].append(numbers[3*n:3*n+3])
                     self.vibdisps.extend(disps)
                 
                 line = inputfile.next()
@@ -577,11 +612,10 @@ class Gaussian(logfileparser.Logfile):
                 toMO = int(p.match(toMO).group())-1 # subtract 1 so that it is an index into moenergies
 
                 percent = self.float(t[1])
-                # Changed: parse the actual coefficient
-                #sqr = percent**2*2 # The fractional contribution of this CI
-                #if percent < 0:
-                #    sqr = -sqr
-                #CIScontrib.append([(fromMO, frommoindex), (toMO, tomoindex), sqr])
+                if len(self.homos)==1:
+                    # For restricted calculations, the following has the
+                    # effect of including each transition twice
+                    percent *= numpy.sqrt(2)
                 CIScontrib.append([(fromMO, frommoindex), (toMO, tomoindex), percent])
                 line = inputfile.next()
             self.etsecs.append(CIScontrib)
@@ -609,14 +643,19 @@ class Gaussian(logfileparser.Logfile):
             self.etrotats = numpy.array(self.etrotats, "d")
 
         # Number of basis sets functions.
+        # Has to deal with lines like:
+        #  NBasis =   434 NAE=    97 NBE=    97 NFC=    34 NFV=     0
+        # and...
+        #  NBasis = 148  MinDer = 0  MaxDer = 0
+        # Although the former is in every file, it doesn't occur before
+        #   the overlap matrix is printed.
         if line[1:7] == "NBasis" or line[4:10] == "NBasis":
 
+            # For counterpoise fragment, skip these lines.
+            if self.counterpoise != 0: return
+
+            # If nbasis was already parsed, check if it changed.
             nbasis = int(line.split('=')[1].split()[0])
-            # Has to deal with lines like:
-            #  NBasis =   434 NAE=    97 NBE=    97 NFC=    34 NFV=     0
-            #     NBasis = 148  MinDer = 0  MaxDer = 0
-            # Although the former is in every file, it doesn't occur before
-            # the overlap matrix is printed
             if hasattr(self, "nbasis"):
                 assert nbasis == self.nbasis
             else:
@@ -625,6 +664,10 @@ class Gaussian(logfileparser.Logfile):
         # Number of linearly-independent basis sets.
         if line[1:7] == "NBsUse":
 
+            # For counterpoise fragment, skip these lines.
+            if self.counterpoise != 0: return
+
+            # If nmo was already parsed, check if it changed.
             nmo = int(line.split('=')[1].split()[0])
             if hasattr(self, "nmo"):
                 assert nmo == self.nmo
@@ -807,16 +850,27 @@ class Gaussian(logfileparser.Logfile):
                 # This was continue before parser refactoring.
                 # continue
 
-            centers = line.split()[1:]
+            centers = map(int, line.split()[1:])
+            centers.sort() # Not always in increasing order
             
             self.coreelectrons = numpy.zeros(self.natom, "i")
 
             for center in centers:
-                while line[:10].find(center) < 0:
+                line = inputfile.next()
+                front = line[:10].strip()
+                while not (front and int(front) == center):
                     line = inputfile.next()
-                
+                    front = line[:10].strip()
                 info = line.split()
-                self.coreelectrons[int(center)-1] = int(info[1]) - int(info[2])
+                self.coreelectrons[center-1] = int(info[1]) - int(info[2])
+
+        # This will be printed for counterpoise calcualtions only.
+        # To prevent crashing, we need to know which fragment is being considered.
+        # Other information is also printed in lines that start like this.
+        if line[1:14] == 'Counterpoise:':
+        
+            if line[42:50] == "fragment":
+                self.counterpoise = int(line[51:54])
 
 
 if __name__ == "__main__":
