@@ -3,7 +3,7 @@ cclib (http://cclib.sf.net) is (c) 2006, the cclib development team
 and licensed under the LGPL (http://www.gnu.org/copyleft/lgpl.html).
 """
 
-__revision__ = "$Revision: 741 $"
+__revision__ = "$Revision: 843 $"
 
 
 import os
@@ -24,6 +24,7 @@ import gamessukparser
 import gaussianparser
 import jaguarparser
 import molproparser
+import orcaparser
 
 
 def openlogfile(filename):
@@ -37,7 +38,7 @@ def openlogfile(filename):
     """
 
     # If there is a single string argument given.
-    if type(filename) == str:
+    if type(filename) in [str, unicode]:
     
         extension = os.path.splitext(filename)[1]
         
@@ -77,8 +78,9 @@ def ccopen(filename, progress=None, loglevel=logging.INFO):
     Inputs:
       filename - the location of a single logfile, or a list of logfiles
 
-    Outputs: one of ADF, GAMESS, GAMESS UK, Gaussian, Jaguar, Molpro, or
-             None (if it cannot figure it out or the file does not exist).
+    Returns:
+      one of ADF, GAMESS, GAMESS UK, Gaussian, Jaguar, Molpro, ORCA, or
+        None (if it cannot figure it out or the file does not exist).
     """
 
     filetype = None
@@ -128,8 +130,11 @@ def ccopen(filename, progress=None, loglevel=logging.INFO):
         elif line[0:8] == "1PROGRAM" and not filetype:
             filetype = molproparser.Molpro
 
-    # Need to close before creating an instance.
-    inputfile.close()
+        elif line.find("O   R   C   A") >= 0:
+            filetype = orcaparser.ORCA
+            break
+
+    inputfile.close() # Need to close before creating an instance
     
     if filetype: # Create an instance of the chosen class
         filetype = apply(filetype, [filename, progress, loglevel])
@@ -144,7 +149,7 @@ def convertor(value, fromunits, tounits):
     """
     
     _convertor = {"eV_to_cm-1": lambda x: x*8065.6,
-                  "hartree_to_eV": lambda x: x*27.2114,
+                  "hartree_to_eV": lambda x: x*27.2113845,
                   "bohr_to_Angstrom": lambda x: x*0.529177,
                   "Angstrom_to_bohr": lambda x: x*1.889716,
                   "nm_to_cm-1": lambda x: 1e7/x,
