@@ -2,7 +2,6 @@ import os
 import sys
 import math
 import numpy
-
 from gausssum.utils import GaussianSpectrum, levelname, percent
 from gausssum.cclib.parser.utils import convertor
 
@@ -61,6 +60,7 @@ def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec)
 
     # Create output directory if necessary
     gaussdir=folder(screen,logfilename)
+    unres = len(logfile.homos) > 1
 
     CIS = logfile.etsecs
 
@@ -93,42 +93,35 @@ def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec)
                     screenCD.append("")
                     # charge density [before, after] for each of the groups for each of the transitions
 
-        majorCIS=[]; minorCIS=[]
-        for i in range(len(logfile.etenergies)):
-            majorCIS.append("")
-            minorCIS.append("")
-
+        majorCIS, minorCIS = [[] for x in logfile.etenergies], [[] for x in logfile.etenergies]
         for i in range(len(logfile.etenergies)): # For each transition
             if orbdata:
-                CD=[]
-                for j in range(NGroups):
-                    CD.append([0,0])
+                CD=[ [0, 0] for x in range(NGroups)]
             totcontribs = 0
             for j in range(len(CIS[i])): # For each contribution
-                mycontrib = CIS[i][j][2]**2 * 2 # don't multiply by two if unrestricted!
+                thisCIS = CIS[i][j]
+                mycontrib = thisCIS[2] ** 2
+                if not unres:
+                    mycontrib *= 2
                 totcontribs += mycontrib # tot up the contribs
                 percontrib = int(mycontrib * 100 + .5)
                 if orbdata:
                     for k in range(NGroups):
                         CD[k][0] = (CD[k][0] + mycontrib *
-                                    contrib[0, CIS[i][j][0][0], k])
+                                    contrib[0, thisCIS[0][0], k])
                         CD[k][1] = (CD[k][1] + mycontrib *
-                                    contrib[0, CIS[i][j][1][0], k])
+                                    contrib[0, thisCIS[i][j][1][0], k])
+                sign = ["", "-"][thisCIS[2] < 0]
+                alphabeta = ["", ""]
+                if unres:
+                    alphabeta = [["(A)", "(B)"][thisCIS[0][1]], ["(A)", "(B)"][thisCIS[1][1]]]
+                CIStext = "%s%s->%s%s (%s%d%%)" % (levelname(thisCIS[0][0], logfile.homos[thisCIS[0][1]]), alphabeta[0],
+                                                   levelname(thisCIS[1][0], logfile.homos[thisCIS[1][1]]), alphabeta[1],
+                                                   sign, percontrib)                    
                 if percontrib >= 10: # Major contributions (>=10%)
-                    sign=""
-                    if CIS[i][j][2]<0:
-                        sign="-"
-                    if majorCIS[i]!="":
-                        majorCIS[i]=majorCIS[i]+", "
-                    majorCIS[i]=majorCIS[i]+levelname(CIS[i][j][0][0],logfile.homos[0])+"->"+levelname(CIS[i][j][1][0],logfile.homos[0])+" ("+sign+str(percontrib)+"%)"
+                    majorCIS[i].append(CIStext)
                 elif percontrib >= 2: #Minor contributions (>=2%)
-                    sign=""
-                    if CIS[i][j][2]<0:
-                        sign="-"
-                    if minorCIS[i]!="":
-                        minorCIS[i]=minorCIS[i]+", "
-                    minorCIS[i]=minorCIS[i]+levelname(CIS[i][j][0][0],logfile.homos[0])+"->"+levelname(CIS[i][j][1][0],logfile.homos[0])+" ("+sign+str(percontrib)+"%)"
-                    
+                    minorCIS[i].append(CIStext)
                     
             if orbdata:
                 for j in range(len(groupname)): # The charge densities are scaled so that they add to one
@@ -145,14 +138,17 @@ def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec)
             temp = [i+1,logfile.etenergies[i],
                     convertor(logfile.etenergies[i],"cm-1","nm"),
                     logfile.etoscs[i],logfile.etsyms[i],
-                    majorCIS[i],minorCIS[i]]
+                    ", ".join(majorCIS[i]),", ".join(minorCIS[i])]
             if orbdata:
                 temp.append(screenCD[i])
             fileoutput += "\t".join(map(str,temp)) + "\n"
 
         screen.write("Writing the transition info to UVData.txt\n")
         outputfile=open(os.path.join(gaussdir,"UVData.txt"),"w")
-        outputfile.write("HOMO is "+str(logfile.homos[0]+1)+"\nNo.\tEnergy (cm-1)\tWavelength (nm)\tOsc. Strength\tSymmetry\tMajor contribs\tMinor contribs")
+        outputfile.write("HOMO is "+str(logfile.homos[0]+1))
+        if unres:
+            outputfile.write("\t%d" % (logfile.homos[1] + 1,))
+        outputfile.write("\nNo.\tEnergy (cm-1)\tWavelength (nm)\tOsc. Strength\tSymmetry\tMajor contribs\tMinor contribs")
         for i in range(NGroups):
             outputfile.write("\t"+groupname[i])
         outputfile.write("\n"+fileoutput)
