@@ -60,6 +60,7 @@ class App:    # This sets up the GUI
         self.scrollbar.config(command=self.txt.yview)
 
         self.screen=Redirect(self.txt.insert,self.root,[END,self.txt]) # Oh yeah! Could I have done it in a more roundabout way?
+        self.screen.write("Support GaussSum by citing:\nN.M. O'Boyle, A.L. Tenderholt and K.M. Langner. J. Comp. Chem. 2008, 29, 839-845.\n")
 
         frame3=Frame(self.frame1,relief=GROOVE,borderwidth=3)
         frame3.pack(side=LEFT,anchor=W)
@@ -376,7 +377,7 @@ class App:    # This sets up the GUI
                 worked = SCF(self.root,self.screen,self.data,int(self.numpts.get()),self.settings['global settings.gnuplot'])
             elif s=="GEOOPT":
                 if self.reparse.get():
-                    data = self.logfile.parse()
+                    self.data = self.logfile.parse()
                 worked = GeoOpt(self.root,self.screen,self.data,int(self.numpts.get()),self.settings['global settings.gnuplot'])                
             elif s=="IR_RAMAN":
                 worked = Vibfreq(self.root,self.screen,self.data,self.logfile.filename,int(self.start.get()),int(self.end.get()),int(self.numpts.get()),float(self.FWHM.get()),self.scale.get(),float(self.scalefactor.get()),self.settings['global settings.gnuplot'])
@@ -561,7 +562,7 @@ class App:    # This sets up the GUI
                     os.mkdir(settingsdir)
                 self.settings={
 ##                               'global settings.filetype':'Gaussian',
-                               'global settings.gnuplot':os.path.join(os.sep,"usr","bin","gnuplot"),
+                               'global settings.gnuplot':os.path.join(installlocation,"gnuplot400","bin","wgnuplot.exe"),
                                'find.text1':'SCF Done',
                                'find.text2':'k 501%k502',
                                'find.text3':'imaginary',
@@ -587,6 +588,6 @@ class App:    # This sets up the GUI
 
  
     def webdocs(self):
-        webbrowser.open(os.path.join(sys.path[0],"Docs","index.html"))
+        webbrowser.open(os.path.join(installlocation,"Docs","index.html"))
     
 
