@@ -14,7 +14,7 @@ from gausssum.utils import levelname
 from gausssum.utils import GaussianSpectrum
 from gausssum.utils import Groups
 
-def Popanalysis(root,screen,logfile,start,end,COOP,FWHM,makeorigin,gnuplotexec):
+def Popanalysis(root,screen,logfile,logfilename,start,end,COOP,FWHM,makeorigin,gnuplotexec):
 
     def DOSconvolute(orb_MPA,evalue):
         """Convolute the DOS spectrum"""
@@ -401,7 +401,7 @@ def Popanalysis(root,screen,logfile,start,end,COOP,FWHM,makeorigin,gnuplotexec):
     groupatoms=[]; groupname=[]; atomorb=[]
 
     # Create the output directory if necessary
-    gaussdir=folder(screen,logfile)
+    gaussdir=folder(screen,logfilename)
 
     # Read in the groups (if they exist!)
     filename = os.path.join(gaussdir,"Groups.txt")
